﻿using Coscine.Configuration;
using FluentMigrator.Runner;
using Microsoft.Extensions.DependencyInjection;

namespace Coscine.Database.Migration
{
    public class Migrator
    {
        // Format: Data Source={DBSERVER};Database={DATABASENAME};User Id={USER};Password={PASSWORD};
        private const string DbDataSourceKey = "coscine/global/db_data_source";
        private const string DbNameKey = "coscine/global/db_name";
        private const string DbUserIdKey = "coscine/global/db_user_id";
        private const string DbPasswordKey = "coscine/global/db_password";

        private readonly IConfiguration _configuration;
        private ServiceProvider _serviceProvider;

        public Migrator(System.Reflection.Assembly targetAssembly)
        {
            _configuration = new ConsulConfiguration();
            SetServiceProvider(targetAssembly, GetDbConnectionString());
        }

        private void SetServiceProvider(System.Reflection.Assembly targetAssembly, string dbConnectionString)
        {
            var serviceProvider = new ServiceCollection()
            // Registration of all FluentMigrator-specific services
            .AddFluentMigratorCore()
            // Configure the runner
            .ConfigureRunner(
                builder => builder
                    // Use SQLServer
                    .AddSqlServer()
                    // The SQLServer connection string  TODO: make configurable (via consul)
                    .WithGlobalConnectionString(dbConnectionString)
                    // Specify the assembly with the migrations
                    .ScanIn(targetAssembly).For.Migrations()
                    // Specify the assembly with the embedded resouces
                    .ScanIn(targetAssembly).For.EmbeddedResources())
                    
            // Enable logging to console in the FluentMigrator way
            .AddLogging(lb => lb.AddFluentMigratorConsole())
            .BuildServiceProvider();

            _serviceProvider = serviceProvider;
        }

        public string GetDbConnectionString()
        {
            var dbDataSource = _configuration.GetString(DbDataSourceKey);
            var dbDatabase = _configuration.GetString(DbNameKey);
            var dbUserId = _configuration.GetString(DbUserIdKey);
            var dbPassword = _configuration.GetString(DbPasswordKey);
            return $"Data Source={dbDataSource}; Database={dbDatabase}; User Id={dbUserId}; Password={dbPassword};";
        }

        public void MigrateUp()
        {
            // Put the database update into a scope to ensure
            // that all resources will be disposed.
            using (var scope = _serviceProvider.CreateScope())
            {
                // Instantiate the runner
                var runner = scope.ServiceProvider.GetRequiredService<IMigrationRunner>();

                // Execute the migrations
                runner.MigrateUp();
            }
        }

        public void RollBack(int steps)
        {
            using (var scope = _serviceProvider.CreateScope())
            {
                var runner = scope.ServiceProvider.GetRequiredService<IMigrationRunner>();
                runner.Rollback(steps);
            }
        }

        // Currently not in use
        internal void MigrateDown(FluentMigrator.IMigration targetMigration)
        {
            using (var scope = _serviceProvider.CreateScope())
            {
                var runner = scope.ServiceProvider.GetRequiredService<IMigrationRunner>();
                runner.Down(targetMigration);
            }
        }
    }
}
